#ifndef WTIMESOLVER_H
#define WTIMESOLVER_H

#include "dart.h"
#include "wViscFluxes.h"
#include <vector>
#include <string>

namespace dart
{

class DART_API TimeSolver
{

private:
    double CFL0;
    unsigned long maxIt;
    double tol;
    unsigned int itFrozenJac0;
    bool initSln;
    double gamma = 1.4;
    double Minf;
    unsigned int nErrors;

    ViscFluxes *SysMatrix;

public:
    TimeSolver(double _CFL0, double _Minf, double Re, unsigned long _maxIt = 100, double _tol = 1e-8, unsigned int _itFrozenJac = 5);
    ~TimeSolver();
    void InitialCondition(size_t iPoint, BLRegion *bl);
    int Integration(size_t iPoint, BLRegion *bl);
    double GetCFL0();
    void SetCFL0(double _CFL0);
    unsigned long GetmaxIt();
    void SetmaxIt(unsigned long _maxIt);
    unsigned int GetitFrozenJac();
    void SetitFrozenJac(unsigned int _itFrozenJac);
    void SetinitSln(bool _initSln);

private:
    double SetTimeStep(double CFL, double dx, double invVel) const;
    double ComputeSoundSpeed(double gradU2) const;
    void ResetSolution(size_t iPoint, BLRegion *bl, std::vector<double> Sln0, bool usePrevPoint = false);
};
} // namespace dart
#endif //WTimeSolver_H